/***************************************************************************
              wavblock.cpp  -  implementation of WAV sound block
             ----------------------------------------------------
    begin                : Sat Jul 29 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "wavblock.h"
#include "sounddevice.h"
#include "dialogprogress.h"
#include "utils.h"

#include <gnome--/main.h>
#include <libgnome/gnome-i18n.h>

#include <iostream.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>						// usleep

#include <strstream>
#include <string>


WAVblock::WAVblock() {
  conversion_subprocess.stderr_data.connect(SigC::slot(this, &WAVblock::received_err));
	conversion_subprocess.stdout_data.connect(SigC::slot(this, &WAVblock::received_data));
	conversion_subprocess.process_exited.connect(SigC::slot(this, &WAVblock::converter_exited));
	conversion_subprocess.data_written.connect(SigC::slot(this, &WAVblock::data_written));
	
	export_progress = 0;
}

WAVblock::WAVblock(const WAVblock &src) : SoundBlock(src) {
  bitsPerSample = src.bitsPerSample;

  conversion_subprocess.stderr_data.connect(SigC::slot(this, &WAVblock::received_err));
	conversion_subprocess.stdout_data.connect(SigC::slot(this, &WAVblock::received_data));
	conversion_subprocess.process_exited.connect(SigC::slot(this, &WAVblock::converter_exited));
	conversion_subprocess.data_written.connect(SigC::slot(this, &WAVblock::data_written));
	
	export_progress = 0;
}


WAVblock::~WAVblock() {
}

bool WAVblock::init_play() {
	if (!assigned) return false;
	
  playedMS = 0;
  lastMS = 0;

  return true;		
}

  	
bool WAVblock::play_block(SoundDevice *sd, string mpg123_path, int from_ms = -1, int to_ms = -1) {
	// set format
	
	if (!assigned || playing) return false;

  play_from_ms = from_ms;
  play_to_ms = to_ms;
	
	playing = true;
	
  sd->close_device();		// reopen sound device - required befor new setting frequncy, format, ...
  sd->open_device();
	
  if (bitsPerSample == 16) sd->set_format(SoundDevice::bit16_le);	// WAV file is always little endian
  else
    if (bitsPerSample == 8) sd->set_format(SoundDevice::bit8);

	sd->set_number_of_channels(channels);				// set channels
	sd->set_frequency(freq);										// set frequency

  buff_size = sd->get_fragment_size();
  buffer = new char[buff_size];

  sdev = sd;

  // create new thread for playing
	start();

	cn = Gtk::Main::timeout.connect(slot(this, &WAVblock::timer_done), 20);

  return true;
}

bool WAVblock::is_changed() {
  return changed;
}

bool WAVblock::scan_file(int b, int e) {	// first sample, last sample
  int num = 2000;												// number of scan points

  if (e < b) {int tmp = b;
  	b = e;
  	e = tmp;
  }

  if (e - b < num) num = e - b;

  // progress dialog
  DialogProgress pd(_("Please wait ..."), _("Scanning WAV file ..."));

  double rat =  double(e - b) / num;

  scannedData.erase(scannedData.begin(), scannedData.end());    // clear scanned data

  int i = 0; // 0
  int val, val2;

  while (i < num) {

    if (i % 10 == 0) {
  	  pd.set_progress(static_cast<float>(i)/num * 100);
  	  while (Gtk::Main::events_pending() > 0) Gtk::Main::iteration();

      if (pd.cancel_state() == true) {
        scannedData.insert(scannedData.end(), num - i, short (32767));
        scan_done.emit();
        return false;
      }
	  }
	
    val = abs(read_sample(static_cast<int>(b + (i * rat)), 0)); 				// Left channel or mono

    for (int x = 1; x <= 32; x++) {
    	val2 = abs(read_sample(static_cast<int>(b + (i * rat) + x), 0)); 				// Left channel or mono
    	if (val2 > val) val = val2;
    }

    if (bitsPerSample == 8) {
      val -= 128;   // 8 bit WAV has unsigned values
      val = val << 8;
    }
    scannedData.push_back(val);

    if (channels > 1) {
      val = abs(read_sample(static_cast<int>(b + (i * rat)), 1)); 									// Right channel if stereo

      for (int x = 1; x <= 32; x++) {
      	val2 = abs(read_sample(static_cast<int>(b + (i * rat) + x), 1)); 				// Left channel or mono
      	if (val2 > val) val = val2;
      }

      if (bitsPerSample == 8) {
        val -= 128;   // 8 bit WAV has unsigned values
        val = val << 8;
      }
    	scannedData.push_back(val);
    }

		i++;
  }

  scan_done.emit();

  return true;
}

int WAVblock::block_length() {	// length of block in ms
	return static_cast<int>(static_cast<double>(num_samples()) / freq * 1000);
}

bool WAVblock::valid() {			// is this soundblock valid?
  return true;
}

string WAVblock::block_info() {	
  string b;
  strstream s;

  s << bitsPerSample;
  s >> b;

  return b + _(" bits/sample");
}

bool WAVblock::assign_file(string fname) {
  char txt[5];

  file.open(fname.c_str(), ios::binary | ios::in);			// open file

//  cerr << "gcount(): " << file.gcount() << endl;

  if (!file.is_open())
  	return false;  						
  		
  fileName = fname;

  // read file info from WAV header
  	
	file.read(txt, 4);						// read header
	txt[4] = '\0';
	
	if (strcmp(txt, "RIFF") != 0) {			// is it RIFF file?
		file.close();
		return false;											// no, it isn't
	}
	
	file.read(txt, 4);						// read length of reminder
	
	file.read(txt, 4);						// read data type
	if (strcmp(txt, "WAVE") != 0) {			// is it WAVE file?
		file.close();
		return false;								// no, it isn't			
	}
		
 	file.read(txt, 4);						// read chunk type

	while (strcmp(txt, "fmt ") != 0) {
		guint32 a;
	  file.read((char *)(&a), sizeof(a));						// read chunk length
  	file.seekg(a, ios::cur);
  	
  	if (file.eof()) {
  		file.close();
  		return false;								// EOF, fmt chunk not found
  	}
  	file.read(txt, 4);						// read chunk type
  }

  // fmt chunk found
  guint32 chunkLen;
  file.read(&chunkLen, sizeof(chunkLen));								// read chunk length

 	guint32 dwAvgBytesPerSec;		// Bytes per second
 	guint16 wBlockAlign;				// Data align
	guint16 wFormatTag;					// Format category
 	guint16 wChannels;					// Number of channels
 	guint32 dwSamplesPerSec;		// Sampling rate
 	guint16 wBitsPerSample;			// Bits per sample

//  file.read((char *)(&wFormatTag), sizeof(wFormatTag));								// read chunk length
  file.read(&wFormatTag, sizeof(wFormatTag));
  file.read((char *)(&wChannels), sizeof(wChannels));
  file.read((char *)(&dwSamplesPerSec), sizeof(dwSamplesPerSec));
  file.read((char *)(&dwAvgBytesPerSec), sizeof(dwAvgBytesPerSec));
  file.read((char *)(&wBlockAlign), sizeof(wBlockAlign));


  if (wFormatTag == 1) {				// PCM format (no compression)	
	  file.read((char *)(&wBitsPerSample), sizeof(wBitsPerSample));								// read chunk length
	}
	else {
	  return false;			// unsupported format
	}
	
	freq = dwSamplesPerSec;
	channels = wChannels;
	bitsPerSample = wBitsPerSample;

 	file.read(txt, 4);						// read chunk type

	while (strcmp(txt, "data") != 0) {
		guint32 a;
	  file.read((char *)(&a), sizeof(a));								// read chunk length
  	file.seekg(a, ios::cur);
  	
  	if (file.eof()) {
  		file.close();
  		return false;											// EOF, data chunk not found
  	}
  	file.read(txt, 4);						// read chunk type
  }

	file.read((char *)(&chunkLen), sizeof(chunkLen));								// read chunk length
	
  begin = file.tellg();						// begin of sound data
  end = begin + chunkLen;

  beginMax = begin;
  endMax = end;

  assigned = true;

  // last modification

  int fd = file.rdbuf()->fd();
  struct stat buf;
  fstat(fd, &buf);
  time_t mt = buf.st_mtime;
  struct tm *t = localtime(&mt);

#ifdef DEBUG_OUTPUT
  cerr << "Modif: " << t->tm_mday << '.' << t->tm_mon+1 << '.' << t->tm_year+1900;
  cerr << ' ' << t->tm_hour << ':' << t->tm_min << ':' << t->tm_sec << endl;
#endif

  return true;
}

bool WAVblock::assigned_file() {
	return assigned;
}

int WAVblock::num_samples() {
  return (end - begin) / (bitsPerSample / 8) / channels;
}

int WAVblock::max_samples() {
  return (endMax - beginMax) / (bitsPerSample / 8) / channels;
}

int WAVblock::read_sample(int pos, int channel) {
  if (!file.is_open()) return 0;

  int p = begin + (pos * channels * (bitsPerSample / 8)) + channel * (bitsPerSample / 8);

  file.seekg(p);

  gint16 i16;
  guint8 i8;

  if (bitsPerSample == 8) {
		file.read((char *)(&i8), sizeof(i8));								// read chunk length
    return i8;
  }
	else {
		file.read((char *)(&i16), sizeof(i16));								// read chunk length
    return i16;
  }
}

void WAVblock::stop_block() {
  if (playing) {
    playing = false;
  }
}

void WAVblock::pause_block() {
  if (playing) paused = true;
}

void WAVblock::unpause_block() {
  if (playing) paused = false;
}

string WAVblock::get_type(){
	return string("WAV PCM");
}


int WAVblock::timer_done() {
	if (!playing) {
		cn.disconnect();
	  block_done.emit();
	  return 0;
	}
//	else
//	  playedMiliSec.emit(playedMS);
	
	return 1;
}

bool WAVblock::save_block_to_stream(ofstream &of, string dirName) {
	of << 'W';			// WAV identification
	
	string relName;
	
	if (fileName.size() > 0 && fileName[0] != '/')
  	relName = fileName;
  else
  	relName = Utils::relativeFile(fileName, dirName);

  of.write(relName.c_str(), relName.size());       // save relative file to project directory
  of << '\0';
//  of.write((char *)(&fileDateTime), sizeof(fileDateTime));

  gint32 tmp32;
  gint16 tmp16;

#ifdef WORDS_BIGENDIAN
  tmp32 = GINT32_TO_LE(freq);
  of.write((char *)(&tmp32), sizeof(tmp32));
#else
  of.write((char *)(&freq), sizeof(freq));
#endif

#ifdef WORDS_BIGENDIAN
  tmp16 = GINT32_TO_LE(channels);
  of.write((char *)(&tmp16), sizeof(tmp16));
#else
  of.write((char *)(&channels), sizeof(channels));
#endif

#ifdef WORDS_BIGENDIAN
  tmp32 = GINT32_TO_LE(begin);
  of.write((char *)(&tmp32), sizeof(tmp32));
#else
  of.write((char *)(&begin), sizeof(begin));
#endif

#ifdef WORDS_BIGENDIAN
  tmp32 = GINT32_TO_LE(end);
  of.write((char *)(&tmp32), sizeof(tmp32));
#else
  of.write((char *)(&end), sizeof(end));
#endif

#ifdef WORDS_BIGENDIAN
  tmp32 = GINT32_TO_LE(beginMax);
  of.write((char *)(&tmp32), sizeof(tmp32));
#else
  of.write((char *)(&beginMax), sizeof(beginMax));
#endif

#ifdef WORDS_BIGENDIAN
  tmp32 = GINT32_TO_LE(endMax);
  of.write((char *)(&tmp32), sizeof(tmp32));
#else
  of.write((char *)(&endMax), sizeof(endMax));
#endif

	// WAV specific
#ifdef WORDS_BIGENDIAN
  tmp16 = GINT32_TO_LE(bitsPerSample);
  of.write((char *)(&tmp16), sizeof(tmp16));
#else
  of.write((char *)(&bitsPerSample), sizeof(bitsPerSample));
#endif

	if (saveScannedData) {
    of << '\1';

    gint32 s = scannedData.size();
#ifdef WORDS_BIGENDIAN
    tmp32 = GINT32_TO_LE(s);
    of.write((char *)(&tmp32), sizeof(tmp32));
#else
    of.write((char *)(&s), sizeof(s));
#endif

    int i = 0;
    while (i < s) {

#ifdef WORDS_BIGENDIAN
      tmp16 = GINT16_TO_LE(scannedData[i]);
      of.write((char *)(&tmp16), sizeof(tmp16));
#else
      of.write((char *)(&scannedData[i]), sizeof(scannedData[i]));
#endif

      i++;
    }
	}
	else {
    of << '\0';
	}

	return true;
}

bool WAVblock::load_block_from_stream(ifstream &is) {
  string q;
  char x;

  is.read(&x, sizeof(x));

  while (x != '\0') {
    q = q + x;
    is.read(&x, sizeof(x));
  }

  fileName = q;
  // convert file name from relative to absolute
  char *cdir = g_get_current_dir();
  fileName = Utils::absoluteFile(q, cdir);
  g_free(cdir);

#ifdef DEBUG_OUTPUT
  cerr << fileName << endl;
#endif

  file.open(fileName.c_str(), ios::binary | ios::in);			// open file

  if (!file.is_open())
  	return false;  						
  		
  assigned = true;

//  is.read(&fileDateTime, sizeof(fileDateTime));

  is.read(&freq, sizeof(freq));
  is.read(&channels, sizeof(channels));
  is.read(&begin, sizeof(begin));
  is.read(&end, sizeof(end));
  is.read(&beginMax, sizeof(beginMax));
  is.read(&endMax, sizeof(endMax));

#ifdef WORDS_BIGENDIAN
  // convert from little endian
  freq = GINT32_FROM_LE(freq);
  channels = GINT16_FROM_LE(channels);
  begin = GINT32_FROM_LE(begin);
  end = GINT32_FROM_LE(end);
  beginMax = GINT32_FROM_LE(beginMax);
  endMax = GINT32_FROM_LE(endMax);
#endif

#ifdef DEBUG_OUTPUT
  cerr << "Freq:  " << freq << endl;
  cerr << "Chan:  " << channels << endl;
  cerr << "Begin:  " << begin << endl;
  cerr << "End:  " << end << endl;
#endif

	// WAV specific
  is.read(&bitsPerSample, sizeof(bitsPerSample));
#ifdef WORDS_BIGENDIAN
  bitsPerSample = GINT16_FROM_LE(bitsPerSample);
#endif

  is.read(&x, sizeof(x));

	if (x == '\1') {              // scanned data present
    gint32 size;                // number of scanned data
		is.read(&size, sizeof(size));
#ifdef WORDS_BIGENDIAN
    size = GINT32_FROM_LE(size);
#endif

    int i = 0;
    gint16 i16;

    while (i < size) {
		  is.read(&i16, sizeof(i16));
#ifdef WORDS_BIGENDIAN
      i16 = GINT16_FROM_LE(i16);
#endif
      scannedData.push_back(i16);
      i++;
    }
	}
	
/*	QFileInfo finfo(file);
	
	if (!error && fileDateTime != finfo.lastModified()) {
		KMsgBox::message(0, _("Information"), _("File ") + ff +
		   _(" was modified.\nPress Ok to rescan file"),
		   KMsgBox::INFORMATION | KMsgBox::DB_FIRST, _("Ok")
		);
	
		file.close();               // close opened file
		assignFile(file.name());    // assign file again - reread header
		scanFile(0, numSamples());  // rescan file
	}

	return error;*/
	
	return true;
}

int WAVblock::get_begin_sample_offset() {
  return (begin - beginMax) / channels / (bitsPerSample / 8);
}

int WAVblock::get_end_sample_offset() {
  return (end - beginMax) / channels / (bitsPerSample / 8);
}

void WAVblock::set_begin_sample_offset(int v) {
  int newBegin = beginMax + v * channels * (bitsPerSample / 8);

  if (newBegin <= end) begin = newBegin;
  block_changed.emit();
}

void WAVblock::set_end_sample_offset(int v) {
  int newEnd = beginMax + v * channels * (bitsPerSample / 8);

  if (newEnd >= begin) end = newEnd;
  block_changed.emit();
}

// thread code
void *WAVblock::main(void *) {
	ifstream istr;		// create new stream for reading (thread ...)
  istr.open(fileName.c_str(), ios::binary | ios::in);			// open file

  int pos_start, pos_end;

  if (play_from_ms > 0) {
    pos_start = begin + (static_cast<int>(play_from_ms / 1000.0  * freq) * (bitsPerSample / 8) * channels);
  }
  else
    pos_start = begin;

  if (play_to_ms > 0) {
    pos_end = begin + (static_cast<int>(play_to_ms / 1000.0 * freq) * (bitsPerSample / 8) * channels);
    if (pos_end > end) pos_end = end;
  }
  else
    pos_end = end;

  // check positions
  if (pos_start >= pos_end) {
	  playing = false;
  	delete[] buffer;
  	buffer = 0;
  	
  	return 0;
	}

  istr.seekg(pos_start, ios::beg);	// play from start

  // calculate number of buffer reads
  num = (pos_end - pos_start) / buff_size;
  rem = (pos_end - pos_start) % buff_size;

	while (num > 0 && playing) {
		while (paused) usleep(50000);
		
    istr.read(buffer, buff_size);
    sdev->play(buffer, buff_size);			

		playedMS = static_cast<int>(static_cast<double>(istr.tellg() - begin) / channels / (bitsPerSample / 8) / freq * 1000);
		num--;
  }

 	if (rem > 0 && playing) {
		while (paused) usleep(50000);
 	
    istr.read(buffer, rem);
    sdev->play(buffer, rem);
 		rem = 0;
 		
		playedMS = static_cast<int>(static_cast<double>(istr.tellg() - begin) / channels / (bitsPerSample / 8) / freq * 1000);
	}

	if (!playing) sdev->reset();	// Immidiately stop playing
	else playing = false;
	
	delete[] buffer;
	buffer = 0;
	
	return 0;
}

void WAVblock::export_WAV(ofstream &ostr, int ch, int f, int bps, DialogProgress *dp,
                         string decoder_path, string conversion_path) {

  int pos = fileName.rfind(G_DIR_SEPARATOR);

  if (pos < 0)
    pos = 0;
  else
    pos++;

	play_to_ms = -1;
	play_from_ms = -1;
  exporting_WAV = true;		// yes, exporting to WAV

  string base_name(fileName, pos, fileName.size());
  dp->set_label(_("Exporting file: ") + base_name);

  if (ch == channels && f == freq && bps == bitsPerSample) {
#ifdef DEBUG_OUTPUT
    cerr << "WAV: export: equal params, no conversion " << endl;
#endif

    // export parameters are equal to block param., only copy data
    int bsize = 4096;
    char *buf = new char[bsize];

    if (!file.is_open()) file.open(fileName.c_str(), ios::binary | ios::in);			// open file
    file.seekg(begin, ios::beg);

    int n = (end - begin) / bsize;
    int rem = (end - begin) % bsize;
    int total = n;

    while (n > 0) {
      file.read(buf, bsize);
      ostr.write(buf, bsize);

      // refresh progress bar
      if (n % 20 == 0) {
        dp->set_progress(100.0f - (static_cast<float>(n) / total * 100));
     	  while (Gtk::Main::events_pending() > 0) Gtk::Main::iteration();
     	
     	  // check whether export was canceled
     	  if (dp->cancel_state()) {
     	    delete[] buf;
          export_done.emit();
          return;
     	  }
      }

      n--;
    }

    if (rem > 0) {
      file.read(buf, rem);
      ostr.write(buf, rem);
      dp->set_progress(100.0f);
    }

    delete[] buf;
    export_done.emit();
  }
  else {
    // different parameters, conversion is needed
#ifdef DEBUG_OUTPUT
    cerr << "WAV: export: non-equal params, conversion required" << endl;
#endif

    export_stream = &ostr;
    export_progress = dp;
    block_num = 0;

    // create vector of conversion parameters
    vector<string> params;

    // input parameters
    params.push_back("-t");   // raw input format
    params.push_back("raw");

    // size of sample
    if (bitsPerSample == 8) {      // 8 bit samples
      params.push_back("-u");      // unsigned
      params.push_back("-b");      // byte
    }
    else {                         // 16 bit samples
      params.push_back("-s");      // signed
      params.push_back("-w");      // word
    }

    // number of channels
    strstream tmp;
    tmp << channels;

    string chanstr;
    tmp >> chanstr;

    params.push_back("-c");
    params.push_back(chanstr);

    // sampling frequency
    strstream tmp2;
    tmp2 << freq;

    string rate;
    tmp2 >> rate;

    params.push_back("-r");
    params.push_back(rate);

    // set input file to stdin
    params.push_back("-");


    // output parameters
    params.push_back("-t");   // raw output format
    params.push_back("raw");

    // size of sample
    if (bps == 8) {                // 8 bit samples
      params.push_back("-u");      // unsigned
      params.push_back("-b");      // byte
    }
    else {                         // 16 bit samples
      params.push_back("-s");      // signed
      params.push_back("-w");      // word
    }

    // number of channels
    strstream tmp3;
    tmp3 << ch;

    string chanstr2;
    tmp3 >> chanstr2;

    params.push_back("-c");
    params.push_back(chanstr2);

    // sampling frequency
    strstream tmp4;
    tmp4 << f;

    string rate2;
    tmp4 >> rate2;

    params.push_back("-r");
    params.push_back(rate2);

    // set output file to stdout
    params.push_back("-");

    conversion_subprocess.start(conversion_path/*"sox"*/, params);

    if (!file.is_open()) file.open(fileName.c_str(), ios::binary | ios::in);			// open file
    file.seekg(begin, ios::beg);
    buff_size = 4096;
    buffer = new char[buff_size];

  	data_written();
  }
}

void WAVblock::converter_exited(int ret) {
#ifdef DEBUG_OUTPUT
	cerr << "WAV block: converter exited:" << ret << endl;
#endif

  if (!export_progress->cancel_state()) export_progress->set_progress(100.0f);

	delete[] buffer;
	buffer = 0;
  export_done.emit();
}

void WAVblock::data_written() {

	int posF = file.tellg();
	
	if (posF < end) {
		if (end - posF >= buff_size) {
    	file.read(buffer, buff_size);
    	conversion_subprocess.write_stdin(buffer, buff_size);
  	}
  	else {
  		int readLen = end - posF;
    	file.read(buffer, readLen);
    	conversion_subprocess.write_stdin(buffer, readLen);
  	}
	}
	else
		conversion_subprocess.close_stdin();
}

void WAVblock::received_data(char *data, int len) {
#ifdef WORDS_BIGENDIAN
  // convert 16 bit data on big endian system to little endian - WAV is little endian
  if (exporting_WAV && bitsPerSample == 16) {			
    gint16 *conv = reinterpret_cast<gint16 *>(data);
    int lng = len / 2;
    for (int i = 0; i < lng; i++) {
      conv[i] = GINT16_TO_LE(conv[i]);
    }
  }
#endif

  export_stream->write(data, len);    // write converted data to stream

  if (block_num % 20 == 0) {
    float percent = static_cast<float>(file.tellg() - begin) / (end - begin) * 100.0f;
    export_progress->set_progress(percent);
// 	  while (Gtk::Main::events_pending() > 0) Gtk::Main::iteration();
 	
    // check whether export was canceled
	  if (export_progress->cancel_state()) {
	    conversion_subprocess.kill_process();
 	  }
  }

  block_num++;
}

void WAVblock::received_err(char *d, int len) {
#ifdef DEBUG_OUTPUT
	int i = 0;
  while(i < len) {
		cerr << d[i++];
  }
  cerr << endl;
#endif
}

void WAVblock::export_MP3(ofstream &ostr, string encoder_name, string encoder_path, int bitrate, int frequency,
		                     ChannelMode channel_mode, bool crc, bool priv, bool original,
		                     bool copyright, vector<string> VBR_options, vector<string> encoder_options,
		                     bool always_process, bool strict_equal, DialogProgress *dp) {
		
  export_stream = &ostr;
  export_progress = dp;
  exporting_WAV = false;    // exporting MP3 not WAV

  vector<string> params;

  // set dialog text to file name
  int pos = fileName.rfind(G_DIR_SEPARATOR);

  if (pos < 0)
    pos = 0;
  else
    pos++;

  play_to_ms = -1;
	play_from_ms = -1;

  string base_name(fileName, pos, fileName.size());
  dp->set_label(_("Exporting file: ") + base_name);

#ifdef DEBUG_OUTPUT
  cerr << "VBR options: ";
  copy(VBR_options.begin(), VBR_options.end(), ostream_iterator<string>(cerr, " "));
  cerr << endl;

  cerr << "encoder options: ";
  copy(encoder_options.begin(), encoder_options.end(), ostream_iterator<string>(cerr, " "));
  cerr << endl;
#endif

  // insert encoder options
  params.insert(params.begin(), encoder_options.begin(), encoder_options.end());

  string conversion_command;

  if (encoder_name == "lame") {
    if (bitrate > 0) {     // CBR
      params.push_back("-b");
      strstream ss;
      string s;

      ss << bitrate;
      ss >> s;
      params.push_back(s);
    }
    else {
      // set VBR option
      params.push_back("-v");
      // insert VBR options
      params.insert(params.end(), VBR_options.begin(), VBR_options.end());
    }

    // CRC option
    if (crc) params.push_back("-p");
    // original option
    if (!original) params.push_back("-o");
    // copyright option
    if (copyright) params.push_back("-c");
    // lame doesn't allow to set private flag

    // set channel mode parameter
    params.push_back("-m");
    switch(channel_mode) {
      case(MODE_STEREO) :
      case(MODE_DUAL)        : params.push_back("s"); break;    // dual flag not supported, set stereo
      case(MODE_JOINT_STEREO): params.push_back("j"); break;
      case(MODE_MONO)        : params.push_back("m"); break;
    }

    // input is RAW
    params.push_back("-r");

    // set input frequency (in kHz)
    params.push_back("-s");
    strstream strs, strs2;
    string st1;
    string st2;
    strs << freq / 1000;
    strs >> st1;
    strs2.width(3);
    strs2.fill('0');
    strs2 << freq % 1000;
    strs2 >> st2;
    params.push_back(st1 + "." + st2);

    // set output frequency (in kHz)
    params.push_back("--resample");
    strstream strstr, strstr2;
    string str1;
    string str2;
    strstr << frequency / 1000;
    strstr >> str1;
    strstr2.width(3);
    strstr2.fill('0');
    strstr2 << frequency % 1000;
    strstr2 >> str2;
    params.push_back(str1 + "." + str2);

    // downmix from stereo to mono if input is stereo and output is mono
    if (channels == 2 && channel_mode == MODE_MONO) params.push_back("-a");

#ifndef WORDS_BIGENDIAN
    // for little endian set byte swapping
    params.push_back("-x");
#endif

    // lame can not handle 8bit samples or mono input with stereo output
    if (bitsPerSample == 8 || (channels == 1 && (channel_mode == MODE_STEREO || channel_mode == MODE_JOINT_STEREO))) {
      // 8 bit samples, mono -> stereo
      // create shell process with colon
      conversion_command = "sh";

      string shell_command;

      shell_command = "sox -t raw";

      // size of sample
      if (bitsPerSample == 8) {
        shell_command += " -u -b";    // unsigned byte
      }
      else {
        shell_command += " -s -w";   // signed word
      }

      // number of channels
      shell_command += " -c";

      if (channels == 1)
        shell_command += " 1";
      else
        shell_command += " 2";

      // sampling frequency
      strstream tmp2;
      tmp2 << freq;
      string rate;
      tmp2 >> rate;

      shell_command += " -r " + rate;

      // set input file to stdin
      shell_command += " -";


      // output parameters
      shell_command += " -t raw";     // raw output format


      // size of sample
      shell_command += " -s -w";      // signed word

      // number of channels
      shell_command += " -c";
      if (channel_mode == MODE_MONO)
        shell_command += " 1";
      else
        shell_command += " 2";

      // output sampling frequency is equal to input frequency
      shell_command += " -r " + rate;

      // set otput file to stdout
      shell_command += " -";

      // create colon
      shell_command += " |";

      shell_command += " " + encoder_path;  // add encoder executable path

      // copy lame options to end of export command
      string old_params;
      unsigned int i = 0;

      while (i < params.size()) {
        old_params += ' ' + params[i];
        i++;
      }
      shell_command += old_params;

      // set stdin input
      shell_command += " -";
      // set stdout output
      shell_command += " -";

      // erase old parameters
      params.erase(params.begin(), params.end());
      // set command option to shell
      params.push_back("-c");
      // set command string
      params.push_back(shell_command);
    }
    else {
      // conversion through sox is not needed
      conversion_command = encoder_path;

      // set stdin input
      params.push_back("-");
      // set stdout output
      params.push_back("-");
    }
  }
  else {
    // other encoder
    export_done.emit();
    return;
  }

  conversion_subprocess.start(conversion_command, params);

  if (!file.is_open()) file.open(fileName.c_str(), ios::binary | ios::in);			// open file
  file.seekg(begin, ios::beg);
  buff_size = 4096;
  buffer = new char[buff_size];

	data_written();
}

//bool WAVblock::play_block_to(SoundDevice *sd, string mpg123_path, int ms) {
//  play_to = true;
//  play_from = false;
//
//  return false;
//}
//
//bool WAVblock::play_block_from(SoundDevice *sd, string mpg123_path, int ms) {
//  play_to = false;
//  play_from = true;
//
//  return false;
//}
//
